/* Comparison of two .BMP images */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>

#define IMAGE1 argv[1]
#define IMAGE2 argv[2]

main(int argc, char *argv[])
{
	int handle1, handle2;
	unsigned int buf[24], image1_header[BUFSIZ], image2_header[BUFSIZ], size_of_header, i, j, *image1, *image2;
	unsigned long int size_of_file, size_of_image, error, x, y, nx, width_of_image, assumed_width_of_image, height_of_image;
	FILE *image_file_1, *image_file_2;

	if (argc != 3) {
		printf("\nUse the following format:\n\ncompress [image_file_1] [image_file_2]\n\n");
		exit(1);
	}
	if ((image_file_1 = fopen(IMAGE1, "rb")) == NULL) {
		fprintf(stderr, "Cannot open file %s\n", IMAGE1);
		exit(1);
	}
	if (fread(buf, sizeof(unsigned int), 24, image_file_1) != 24) {
		fprintf(stderr, "Error reading file %s\n", IMAGE1);
		exit(1);
	}
	if (buf[0] != 19778) {
		printf("%s is not a .BMP file", IMAGE1);
		exit(0);
	}
	if (buf[23] != 2) {
		printf("%s is not a black and white image", IMAGE1);
		exit(0);
	}
	size_of_file = buf[2] * 65536 + buf[1];
	size_of_header = buf[5];
	size_of_image = size_of_file - size_of_header;
	width_of_image = buf[9];
	if (width_of_image % 32 != 0)
		assumed_width_of_image = width_of_image + 32 - (width_of_image % 32);
	else assumed_width_of_image = width_of_image;
	height_of_image = buf[11];
	if (fseek(image_file_1, 0, SEEK_SET) != 0) {
		fprintf(stderr, "Error with file %s\n", IMAGE1);
		exit(1);
	}
	if (fread(image1_header, sizeof(unsigned int), size_of_header/2, image_file_1) != size_of_header/2) {
		fprintf(stderr, "Cannot read header from %s\n", IMAGE1);
		exit(1);
	}
	if ((image_file_2 = fopen(IMAGE2, "rb")) == NULL) {
		fprintf(stderr, "Cannot open file %s\n", IMAGE2);
		exit(1);
	}
	if (fread(image2_header, sizeof(unsigned int), size_of_header/2, image_file_2) != size_of_header/2) {
		fprintf(stderr, "Cannot read header from %s\n", IMAGE2);
		exit(1);
	}
	if (image2_header[0] != 19778) {
		printf("%s is not a .BMP file", IMAGE2);
		exit(0);
	}
	if (image2_header[23] != 2) {
		printf("%s is not a black and white image", IMAGE2);
		exit(0);
	}
	if ((handle1 = open(IMAGE1, O_RDONLY)) == -1) {
		printf("Cannot open file %s\n", IMAGE1);
		exit(1);
	}
	if ((handle2 = open(IMAGE2, O_RDONLY)) == -1) {
		printf("Cannot open file %s\n", IMAGE2);
		exit(1);
	}
	if (filelength(handle1) != filelength(handle2)) {
		printf("%s and %s are different sized files\n", IMAGE1, IMAGE2);
		exit(0);
	}
	close(handle1);
	close(handle2);
	for (i = 0; i < size_of_header/2; i++)
		if (image1_header[i] != image2_header[i]) {
			printf("%s and %s have different headers\n\n", IMAGE1, IMAGE2);
			exit(0);
		}
	if ((image1 = (unsigned int *) calloc(size_of_image/2, sizeof(unsigned int))) == NULL) {
		fprintf(stderr, "Unable to assign enough memory for %s\n", IMAGE1);
		exit(1);
	}
	if ((image2 = (unsigned int *) calloc(size_of_image/2, sizeof(unsigned int))) == NULL) {
		fprintf(stderr, "Unable to assign enough memory for %s\n", IMAGE2);
		exit(1);
	}
	if (fread(image1, size_of_image/2, sizeof(unsigned int), image_file_1) == NULL)
		fprintf(stderr, "Error reading file %s\n", IMAGE1);
	if (fread(image2, size_of_image/2, sizeof(unsigned int), image_file_2) == NULL)
		fprintf(stderr, "Error reading file %s\n", IMAGE1);
	fclose(image_file_1);
	fclose(image_file_2);
	error = 0;
	for (y = 0; y < height_of_image * assumed_width_of_image; y+=assumed_width_of_image)
		for (x = y; x < y + width_of_image; x++) {
			nx = (x%16 < 8) ? x+8 : x-8;
			error += ((image1[nx/16] >> (15 - nx%16)) ^ (image2[nx/16] >> (15 - nx%16))) & 1;
		}
	free(image1);
	free(image2);
	printf("total number of pixels: %ld\n", width_of_image * height_of_image);
	printf("number of incorrect pixels: %ld\n", error);
	printf("percentage error: %u%%\n\n", (100 * error + (width_of_image * height_of_image)/2) / (width_of_image * height_of_image));
	return 0;
}
